@@ -1,7 +1,7 @@
 /*
- * $Id: invoker.c 8428 2006-11-28 21:55:13Z guillem $
+ * $Id$
  *
- * Copyright (C) 2005, 2006 Nokia Corporation
+ * Copyright (C) 2005, 2006, 2007, 2008 Nokia Corporation
  *
  * Author: Guillem Jover <guillem.jover@nokia.com>
  *
@@ -30,6 +30,7 @@
 #include <signal.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/time.h>
 #include <sys/resource.h>
 #include <unistd.h>
 #include <errno.h>
@@ -37,6 +38,7 @@
 #include "config.h"
 #include "report.h"
 #include "invokelib.h"
+#include "search.h"
 
 #define DEFAULT_DELAY 0
 
@@ -85,25 +87,33 @@ sigs_restore(void)
   sigs_set(&sig);
 }
 
+static void
+disable_self_core_dump(void)
+{
+  struct rlimit rlim = { 0, 0 };
+
+  if (setrlimit(RLIMIT_CORE, &rlim) != 0)
+    warning("could not disable self core dumping");
+}
+
 static bool
 invoke_recv_ack(int fd)
 {
-  uint32_t msg;
+  uint32_t action;
 
   /* Revceive ACK. */
-  invoke_recv_msg(fd, &msg);
+  invoke_recv_msg(fd, &action);
 
-  if (msg != INVOKER_MSG_ACK)
-    die(1, "receiving wrong ack (%08x)\n", msg);
+  if (action != INVOKER_MSG_ACK)
+    die(1, "receiving wrong ack (%08x)\n", action);
   else
     return true;
 }
 
 static int
-invoker_init(unsigned int delay)
+invoker_init(void)
 {
   int fd;
-  int options = 0;
   struct sockaddr_un sun;
 
   fd = socket(PF_UNIX, SOCK_STREAM, 0);
@@ -116,14 +126,18 @@ invoker_init(unsigned int delay)
   if (connect(fd, (struct sockaddr *)&sun, sizeof(sun)) < 0)
     die(1, "connecting to the launcher\n");
 
-  if (!delay)
-    options |= INVOKER_MSG_MAGIC_OPTION_WAIT;
+  return fd;
+}
 
+static bool
+invoker_send_magic(int fd, int options)
+{
   /* Send magic. */
   invoke_send_msg(fd, INVOKER_MSG_MAGIC | INVOKER_MSG_MAGIC_VERSION | options);
+
   invoke_recv_ack(fd);
 
-  return fd;
+  return true;
 }
 
 static bool
@@ -192,17 +206,17 @@ invoker_send_end(int fd)
 static bool
 invoker_recv_pid(int fd)
 {
-  uint32_t msg;
+  uint32_t action, pid;
 
   /* Receive action. */
-  invoke_recv_msg(fd, &msg);
+  invoke_recv_msg(fd, &action);
 
-  if (msg != INVOKER_MSG_PID)
-    die(1, "receiving bad pid (%08x)\n", msg);
+  if (action != INVOKER_MSG_PID)
+    die(1, "receiving bad pid (%08x)\n", action);
 
   /* Receive pid. */
-  invoke_recv_msg(fd, &msg);
-  invoked_pid = msg;
+  invoke_recv_msg(fd, &pid);
+  invoked_pid = pid;
 
   return true;
 }
@@ -210,18 +224,18 @@ invoker_recv_pid(int fd)
 static int
 invoker_recv_exit(int fd)
 {
-  uint32_t msg;
+  uint32_t action, status;
 
   /* Receive action. */
-  invoke_recv_msg(fd, &msg);
+  invoke_recv_msg(fd, &action);
 
-  if (msg != INVOKER_MSG_EXIT)
-    die(1, "receiving bad exit status (%08x)\n", msg);
+  if (action != INVOKER_MSG_EXIT)
+    die(1, "receiving bad exit status (%08x)\n", action);
 
   /* Receive status. */
-  invoke_recv_msg(fd, &msg);
+  invoke_recv_msg(fd, &status);
 
-  return msg;
+  return status;
 }
 
 static uint32_t
@@ -328,6 +342,7 @@ main(int argc, char *argv[])
   char *launch = NULL;
   char *delay_str = NULL;
   unsigned int delay;
+  int magic_options = 0;
 
   if (strstr(argv[0], PROG_NAME))
   {
@@ -347,7 +362,12 @@ main(int argc, char *argv[])
 	usage(1);
       else
       {
-	prog_name = argv[i];
+	char *period;
+
+	prog_name = search_program(argv[i]);
+	period = strstr(argv[i], ".launch");
+	if (period)
+		*period = '\0';
 	prog_argc = argc - i;
 	prog_argv = &argv[i];
 	break;
@@ -360,7 +380,7 @@ main(int argc, char *argv[])
      * Do not try to parse any arguments. */
     if (asprintf(&launch, "%s.launch", argv[0]) < 0)
       die(1, "allocating program name buffer");
-    prog_name = launch;
+    prog_name = search_program(launch);
     prog_argc = argc;
     prog_argv = argv;
 
@@ -382,7 +402,12 @@ main(int argc, char *argv[])
 
   delay = get_delay(delay_str);
 
-  fd = invoker_init(delay);
+  if (!delay)
+    magic_options |= INVOKER_MSG_MAGIC_OPTION_WAIT;
+
+  fd = invoker_init();
+
+  invoker_send_magic(fd, magic_options);
   invoker_send_name(fd, prog_argv[0]);
   invoker_send_exec(fd, prog_name);
   invoker_send_args(fd, prog_argc, prog_argv);
@@ -391,6 +416,8 @@ main(int argc, char *argv[])
 
   if (launch)
     free(launch);
+  if (prog_name)
+    free(prog_name);
 
   if (delay)
   {
@@ -406,6 +433,11 @@ main(int argc, char *argv[])
     debug("waiting for invoked program to exit\n");
 
     invoker_recv_pid(fd);
+
+    /* We should disable self core dumps as late as possible, so that we have
+     * a chance to dump on our own bugs. */
+    disable_self_core_dump();
+
     sigs_init();
     status = invoker_recv_exit(fd);
     sigs_restore();
