@@ -1,10 +1,10 @@
 /*
- * $Id: launcher.c 7960 2006-11-13 13:15:31Z guillem $
+ * $Id$
  *
- * Copyright (C) 2005, 2006 Nokia Corporation
+ * Copyright (C) 2005, 2006, 2007, 2008 Nokia Corporation
  *
  * Authors: Michael Natterer <mitch@imendio.com>
- *	    Guillem Jover <guillem.jover@nokia.com>
+ *          Guillem Jover <guillem.jover@nokia.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,12 +43,14 @@
 
 #include "report.h"
 #include "invokelib.h"
+#include "comm_msg.h"
 #include "comm_dbus.h"
 #include "booster.h"
 #include "prog.h"
 
 /* FIXME: Should go into '/var/run/'. */
 #define LAUNCHER_PIDFILE "/tmp/"PROG_NAME".pid"
+#define LAUNCHER_STATEFILE "/tmp/"PROG_NAME".state"
 
 typedef struct
 {
@@ -65,9 +67,11 @@ typedef struct
 } kindergarten_t;
 
 static char *pidfilename = LAUNCHER_PIDFILE;
+static char *statefilename = LAUNCHER_STATEFILE;
 static pid_t is_parent = 1;
 static volatile bool sigchild_catched = false;
 static volatile bool sighup_catched = false;
+static volatile bool sigreexec_catched = false;
 static bool send_app_died = false;
 
 #define OOM_ENABLE "0"
@@ -110,7 +114,7 @@ rise_oom_defense(pid_t pid)
     char pidstr[20];
 
     snprintf(pidstr, sizeof(pidstr), "%d", pid);
-    execl(MAEMO_DEFENDER, MAEMO_DEFENDER, pidstr, NULL);
+    execl(DEFENDER, DEFENDER, pidstr, NULL);
     _exit(1);
   }
 
@@ -118,7 +122,7 @@ rise_oom_defense(pid_t pid)
     return true;
   else
   {
-    error("rising the oom shield for pid=%d status=%d\n", pid, status);
+    warning("rising the oom shield for pid=%d status=%d\n", pid, status);
     return false;
   }
 }
@@ -128,8 +132,6 @@ launch_process(prog_t *prog)
 {
   int cur_prio;
 
-  load_main(prog);
-
   /* Possibly restore process priority. */
   errno = 0;
   cur_prio = getpriority(PRIO_PROCESS, 0);
@@ -140,6 +142,8 @@ launch_process(prog_t *prog)
   if (prog->prio >= 0)
     oom_unprotect();
 
+  load_main(prog);
+
   print_prog_env_argv(prog);
 
   debug("launching process: '%s'\n", prog->filename);
@@ -150,7 +154,7 @@ launch_process(prog_t *prog)
   report_set_output(report_none);
 #endif
 
-  _exit(prog->entry(prog->argc, prog->argv));
+  exit(prog->entry(prog->argc, prog->argv));
 }
 
 static int
@@ -164,7 +168,6 @@ invoked_init(void)
     die(1, "opening invoker socket\n");
 
   unlink(INVOKER_SOCK);
-  umask(022);
 
   sun.sun_family = AF_UNIX;
   strcpy(sun.sun_path, INVOKER_SOCK);
@@ -181,27 +184,27 @@ invoked_init(void)
 static bool
 invoked_get_magic(int fd, prog_t *prog)
 {
-  uint32_t msg;
+  uint32_t magic;
 
   /* Receive the magic. */
-  invoke_recv_msg(fd, &msg);
-  if ((msg & INVOKER_MSG_MASK) == INVOKER_MSG_MAGIC)
+  invoke_recv_msg(fd, &magic);
+  if ((magic & INVOKER_MSG_MASK) == INVOKER_MSG_MAGIC)
   {
-    if ((msg & INVOKER_MSG_MAGIC_VERSION_MASK) == INVOKER_MSG_MAGIC_VERSION)
+    if ((magic & INVOKER_MSG_MAGIC_VERSION_MASK) == INVOKER_MSG_MAGIC_VERSION)
       invoke_send_msg(fd, INVOKER_MSG_ACK);
     else
     {
-      error("receiving bad magic version (%08x)\n", msg);
+      error("receiving bad magic version (%08x)\n", magic);
       return false;
     }
   }
   else
   {
-    error("receiving bad magic (%08x)\n", msg);
+    error("receiving bad magic (%08x)\n", magic);
     return false;
   }
 
-  prog->options = msg & INVOKER_MSG_MAGIC_OPTION_MASK;
+  prog->options = magic & INVOKER_MSG_MAGIC_OPTION_MASK;
 
   return true;
 }
@@ -244,14 +247,14 @@ static bool
 invoked_get_args(int fd, prog_t *prog)
 {
   int i;
-  uint32_t msg;
+  uint32_t argc;
   size_t size;
 
   /* Get argc. */
-  invoke_recv_msg(fd, &msg);
-  prog->argc = msg;
-  size = msg * sizeof(char *);
-  if (size < msg)
+  invoke_recv_msg(fd, &argc);
+  prog->argc = argc;
+  size = argc * sizeof(char *);
+  if (size < argc)
   {
     error("on buggy or malicious invoker code, heap overflow avoided\n");
     return false;
@@ -282,10 +285,10 @@ invoked_get_args(int fd, prog_t *prog)
 static bool
 invoked_get_prio(int fd, prog_t *prog)
 {
-  uint32_t msg;
+  uint32_t prio;
 
-  invoke_recv_msg(fd, &msg);
-  prog->prio = msg;
+  invoke_recv_msg(fd, &prio);
+  prog->prio = prio;
 
   invoke_send_msg(fd, INVOKER_MSG_ACK);
 
@@ -293,25 +296,31 @@ invoked_get_prio(int fd, prog_t *prog)
 }
 
 static bool
-invoked_send_pid(int fd, pid_t pid)
+invoked_send_action(int fd, int action, int value)
 {
-  invoke_send_msg(fd, INVOKER_MSG_PID);
-  invoke_send_msg(fd, pid);
+  invoke_send_msg(fd, action);
+  invoke_send_msg(fd, value);
 
   return true;
 }
 
+static bool
+invoked_send_pid(int fd, pid_t pid)
+{
+  return invoked_send_action(fd, INVOKER_MSG_PID, pid);
+}
+
 static bool
 invoked_get_actions(int fd, prog_t *prog)
 {
   while (1)
   {
-    uint32_t msg;
+    uint32_t action;
 
     /* Get the action. */
-    invoke_recv_msg(fd, &msg);
+    invoke_recv_msg(fd, &action);
 
-    switch (msg)
+    switch (action)
     {
     case INVOKER_MSG_EXEC:
       invoked_get_exec(fd, prog);
@@ -330,7 +339,7 @@ invoked_get_actions(int fd, prog_t *prog)
 
       return true;
     default:
-      error("receiving invalid action (%08x)\n", msg);
+      error("receiving invalid action (%08x)\n", action);
       return false;
     }
   }
@@ -339,10 +348,14 @@ invoked_get_actions(int fd, prog_t *prog)
 static bool
 invoked_send_exit(int fd, int status)
 {
-  invoke_send_msg(fd, INVOKER_MSG_EXIT);
-  invoke_send_msg(fd, status);
+  return invoked_send_action(fd, INVOKER_MSG_EXIT, status);
+}
 
-  return true;
+static bool
+invoked_send_fake_exit(int fd)
+{
+  /* Send a fake exit code, so the invoker does not wait for us. */
+  return invoked_send_exit(fd, 0);
 }
 
 static void
@@ -350,6 +363,7 @@ clean_daemon(int signal)
 {
   if (is_parent)
   {
+    unlink(statefilename);
     unlink(pidfilename);
     unlink(INVOKER_SOCK);
     killpg(0, signal);
@@ -358,75 +372,90 @@ clean_daemon(int signal)
   exit(0);
 }
 
-static kindergarten_t *
-alloc_childs(int n)
+static inline void
+child_clone(child_t *a, child_t *b)
 {
-  kindergarten_t *childs;
+  *a = *b;
+}
 
-  childs = malloc(sizeof(kindergarten_t));
-  if (!childs)
+static void
+child_destroy(child_t *child)
+{
+  child->pid = 0;
+  close(child->sock);
+  child->sock = 0;
+  free(child->name);
+  child->name = NULL;
+}
+
+static kindergarten_t *
+kindergarten_new(int n)
+{
+  kindergarten_t *kg;
+
+  kg = malloc(sizeof(kindergarten_t));
+  if (!kg)
   {
     error("allocating a kindergarten\n");
     return NULL;
   }
 
-  childs->used = 0;
-  childs->n = n;
-  childs->list = calloc(n, sizeof(child_t));
-  if (!childs->list)
+  kg->used = 0;
+  kg->n = n;
+  kg->list = calloc(n, sizeof(child_t));
+  if (!kg->list)
   {
     error("allocating a child list\n");
+    free(kg);
     return NULL;
   }
 
-  return childs;
+  return kg;
 }
 
 static bool
-grow_childs(kindergarten_t *childs)
+kindergarten_grow(kindergarten_t *kg)
 {
-  int halfsize = childs->n * sizeof(child_t);
+  int halfsize = kg->n * sizeof(child_t);
   int size = halfsize * 2;
   void *p;
 
-  p = realloc(childs->list, size);
+  p = realloc(kg->list, size);
   if (!p)
     return false;
 
-  childs->list = p;
-  memset(childs->list + childs->n, 0, halfsize);
+  kg->list = p;
+  memset(kg->list + kg->n, 0, halfsize);
 
-  childs->n *= 2;
+  kg->n *= 2;
 
   return true;
 }
 
 static bool
-release_childs(kindergarten_t *childs)
+kindergarten_destroy(kindergarten_t *kg)
 {
   int i;
-  child_t *list = childs->list;
+  child_t *list = kg->list;
 
-  for (i = 0; i < childs->used; i++)
-  {
-    close(list[i].sock);
-    free(list[i].name);
-  }
+  for (i = 0; i < kg->n; i++)
+    if (list[i].pid)
+      child_destroy(&list[i]);
 
-  childs->used = 0;
-  childs->n = 0;
-  free(childs->list);
+  kg->used = 0;
+  kg->n = 0;
+  free(kg->list);
 
   return true;
 }
 
 static int
-get_child_slot_by_pid(kindergarten_t *childs, pid_t pid)
+kindergarten_get_child_slot_by_pid(kindergarten_t *kg, pid_t pid)
 {
-  child_t *list = childs->list;
+  child_t *list = kg->list;
   int i;
 
-  for (i = 0; i < childs->n; i++)
+  for (i = 0; i < kg->n; i++)
     if (list[i].pid == pid)
       return i;
 
@@ -434,41 +463,27 @@ get_child_slot_by_pid(kindergarten_t *childs, pid_t pid)
 }
 
 static bool
-invoked_send_fake_exit(int sock)
-{
-  /* Send a fake exit code, so the invoker does not wait for us. */
-  invoked_send_exit(sock, 0);
-  close(sock);
-
-  return true;
-}
-
-static bool
-assign_child_slot(kindergarten_t *childs, child_t *child)
+kindergarten_insert_child(kindergarten_t *kg, child_t *child)
 {
   int id;
 
-  if (childs->used == childs->n && !grow_childs(childs))
+  if (kg->used == kg->n && !kindergarten_grow(kg))
   {
     error("cannot make a bigger kindergarten, not tracking child %d\n",
 	  child->pid);
-    invoked_send_fake_exit(child->sock);
     return false;
   }
 
-  id = get_child_slot_by_pid(childs, 0);
+  id = kindergarten_get_child_slot_by_pid(kg, 0);
   if (id < 0)
   {
     error("this cannot be happening! no free slots on the kindergarten,\n"
 	  "not tracking child %d\n", child->pid);
-    invoked_send_fake_exit(child->sock);
     return false;
   }
 
-  childs->list[id].name = child->name;
-  childs->list[id].sock = child->sock;
-  childs->list[id].pid = child->pid;
-  childs->used++;
+  child_clone(&kg->list[id], child);
+  kg->used++;
 
   return true;
 }
@@ -486,27 +501,21 @@ child_died_painfully(int status)
 }
 
 static bool
-release_child_slot(kindergarten_t *childs, pid_t pid, int status)
+kindergarten_release_child(kindergarten_t *kg, pid_t pid, int status)
 {
-  int id = get_child_slot_by_pid(childs, pid);
+  int id = kindergarten_get_child_slot_by_pid(kg, pid);
 
   if (id >= 0)
   {
-    child_t *child = &childs->list[id];
+    child_t *child = &kg->list[id];
 
     if (send_app_died && child_died_painfully(status))
       comm_send_app_died(child->name, pid, status);
 
     invoked_send_exit(child->sock, status);
 
-    close(child->sock);
-    free(child->name);
-
-    child->name = NULL;
-    child->sock = 0;
-    child->pid = 0;
-
-    childs->used--;
+    child_destroy(child);
+    kg->used--;
   }
   else
     info("no child %i found in the kindergarten.\n", pid);
@@ -515,27 +524,125 @@ release_child_slot(kindergarten_t *childs, pid_t pid, int status)
 }
 
 static void
-clean_childs(kindergarten_t *childs)
+kindergarten_reap_childs(kindergarten_t *kg)
 {
   int status;
-  char *cause;
   pid_t childpid;
 
   while ((childpid = waitpid(-1, &status, WNOHANG)) > 0)
   {
-    release_child_slot(childs, childpid, status);
+    kindergarten_release_child(kg, childpid, status);
 
     if (WIFEXITED(status))
-      asprintf(&cause, "exit()=%d", WEXITSTATUS(status));
+      info("child (pid=%d) terminated due to exit()=%d\n", childpid,
+	   WEXITSTATUS(status));
     else if (WIFSIGNALED(status))
-      asprintf(&cause, "signal=%d", WTERMSIG(status));
-
-    info("child (pid=%d) exited due to %s\n", childpid, cause);
-
-    free(cause);
+      info("child (pid=%d) terminated due to signal=%d\n", childpid,
+	   WTERMSIG(status));
+    else
+      info("child (pid=%d) terminated due to unknown status=%d\n", status);
   }
 }
 
+/* Persistence support */
+
+#define LAUNCHER_STATE_SIG 0x30534c4d
+
+static bool
+store_state(kindergarten_t *childs, int invoker_fd)
+{
+  int i;
+  int fd;
+  child_t *list = childs->list;
+  comm_msg_t *msg;
+
+  unlink(statefilename);
+
+  fd = open(statefilename, O_WRONLY | O_CREAT, 0644);
+  if (fd < 0)
+  {
+    error("opening persistence file '%s'\n", statefilename);
+    return false;
+  }
+
+  msg = comm_msg_new(512, 0);
+
+  comm_msg_put_magic(msg, LAUNCHER_STATE_SIG);
+
+  comm_msg_put_int(msg, invoker_fd);
+  comm_msg_put_int(msg, childs->used);
+
+  for (i = 0; i < childs->n; i++)
+  {
+    if (!list[i].pid)
+      continue;
+
+    comm_msg_put_int(msg, list[i].pid);
+    comm_msg_put_int(msg, list[i].sock);
+    comm_msg_put_str(msg, list[i].name);
+  }
+
+  comm_msg_send(fd, msg);
+
+  comm_msg_destroy(msg);
+  close(fd);
+
+  return true;
+}
+
+static kindergarten_t *
+load_state(int *invoker_fd)
+{
+  int i;
+  int fd;
+  uint32_t w, magic;
+  const char *s;
+  kindergarten_t *childs;
+  child_t *list;
+  comm_msg_t *msg;
+
+  fd = open(statefilename, O_RDONLY);
+  if (fd < 0)
+  {
+    if (errno != ENOENT)
+      error("opening persistence file '%s'\n", statefilename);
+    return NULL;
+  }
+
+  msg = comm_msg_new(512, 0);
+  comm_msg_recv(fd, msg);
+
+  close(fd);
+
+  comm_msg_get_magic(msg, &magic);
+  if (LAUNCHER_STATE_SIG != magic)
+  {
+    error("wrong signature on persistence file '%s'\n", statefilename);
+    comm_msg_destroy(msg);
+
+    return NULL;
+  }
+
+  comm_msg_get_int(msg, invoker_fd);
+
+  comm_msg_get_int(msg, &w);
+  childs = kindergarten_new(w);
+  childs->used = w;
+  list = childs->list;
+
+  for (i = 0; i < childs->used; i++)
+  {
+    comm_msg_get_int(msg, &list[i].pid);
+    comm_msg_get_int(msg, &list[i].sock);
+    comm_msg_get_str(msg, &s);
+    list[i].name = strdup(s);
+  }
+
+  comm_msg_destroy(msg);
+
+  return childs;
+}
+
 static void
 sigchild_handler(int sig)
 {
@@ -548,6 +655,12 @@ sighup_handler(int sig)
   sighup_catched = true;
 }
 
+static void
+sigreexec_handler(int sig)
+{
+  sigreexec_catched = true;
+}
+
 static void
 sigs_init(void)
 {
@@ -568,6 +681,9 @@ sigs_init(void)
 
   sig.sa_handler = sighup_handler;
   sigaction(SIGHUP, &sig, NULL);
+
+  sig.sa_handler = sigreexec_handler;
+  sigaction(SIGUSR1, &sig, NULL);
 }
 
 static void
@@ -584,12 +700,14 @@ sigs_restore(void)
   sigaction(SIGTERM, &sig, NULL);
   sigaction(SIGCHLD, &sig, NULL);
   sigaction(SIGHUP, &sig, NULL);
+  sigaction(SIGUSR1, &sig, NULL);
 }
 
 static void
 sigs_interrupt(int flag)
 {
   siginterrupt(SIGCHLD, flag);
+  siginterrupt(SIGUSR1, flag);
 }
 
 static void
@@ -627,11 +745,13 @@ daemonize(void)
     die(1, "forking while daemonizing");
   else if (pid)
     _exit(0);
+}
 
+static void
+fs_init(void)
+{
+  umask(022);
   chdir("/");
-
-  /* Protect us from the oom monster. */
-  rise_oom_defense(getpid());
 }
 
 static void
@@ -665,7 +785,9 @@ usage(int status)
 	 "Options:\n"
 	 "  --daemon            Fork and go into the background.\n"
 	 "  --pidfile FILE      Specify a different pid file (default %s).\n"
-	 "  --booster MODULE    Specify a booster module to use (default '%s').\n"
+	 "  --booster MODULE,...\n"
+	 "                      Specify a comma-separated list of boosters to use"
+	 "                      (default '%s').\n"
 	 "  --send-app-died     Send application died signal.\n"
 	 "  --quiet             Do not print anything.\n"
 	 "  --version           Print program version.\n"
@@ -674,7 +796,7 @@ usage(int status)
 	 "Use the invoker to start a <shared object> from the launcher.\n"
 	 "Where <shared object> is a binary including a 'main' symbol.\n"
 	 "Note that the binary needs to be linked with -shared or -pie.\n",
-	 PROG_NAME, LAUNCHER_PIDFILE, MAEMO_BOOSTER_DEFAULT);
+	 PROG_NAME, LAUNCHER_PIDFILE, BOOSTER_DEFAULT);
 
   exit(status);
 }
@@ -682,13 +804,14 @@ usage(int status)
 int
 main(int argc, char *argv[])
 {
-  booster_t booster = { .name = MAEMO_BOOSTER_DEFAULT };
-  kindergarten_t *childs;
+  booster_t *boosters = NULL;
+  kindergarten_t *kg;
   const int initial_child_slots = 64;
   int i;
   int fd;
   bool daemon = false;
   bool quiet = false;
+  bool upgrading = false;
 
   /*
    * Parse arguments.
@@ -707,7 +830,7 @@ main(int argc, char *argv[])
     else if (strcmp(argv[i], "--booster") == 0)
     {
       if (argv[++i])
-	booster.name = argv[i];
+	boosters_alloc(&boosters, argv[i]);
     }
     else if (strcmp(argv[i], "--send-app-died") == 0)
       send_app_died = true;
@@ -719,29 +842,44 @@ main(int argc, char *argv[])
       usage(-1);
   }
 
+  if (!boosters)
+    boosters_alloc(&boosters, BOOSTER_DEFAULT);
+
   /*
    * Daemon initialization.
    */
-  booster_module_load(&booster);
-
-  booster.state = booster.api->booster_preinit(&argc, &argv);
+  boosters_load(boosters, &argc, &argv);
 
   sigs_init();
   env_init();
+  fs_init();
 
-  /* Setup child tracking. */
-  childs = alloc_childs(initial_child_slots);
+  kg = load_state(&fd);
 
-  /* Setup the conversation channel with the invoker. */
-  fd = invoked_init();
+  if (kg)
+    upgrading = true;
+  else
+  {
+    /* Setup child tracking. */
+    kg = kindergarten_new(initial_child_slots);
 
-  if (daemon)
-    daemonize();
+    /* Setup the conversation channel with the invoker. */
+    fd = invoked_init();
+
+    if (daemon)
+      daemonize();
+  }
+
+  /* Protect us from the oom monster. */
+  rise_oom_defense(getpid());
 
   if (quiet)
     console_quiet();
 
-  create_pidfile();
+  if (upgrading)
+    info("restored persitent state\n");
+  else
+    create_pidfile();
 
   /*
    * Application invokation loop.
@@ -761,16 +899,28 @@ main(int argc, char *argv[])
     /* Handle signals received. */
     if (sigchild_catched)
     {
-      clean_childs(childs);
+      kindergarten_reap_childs(kg);
       sigchild_catched = false;
     }
 
     if (sighup_catched)
     {
-      booster.api->booster_reload(booster.state);
+      boosters_reload(boosters);
       sighup_catched = false;
     }
 
+    if (sigreexec_catched)
+    {
+      info("storing kindergarten state\n");
+      store_state(kg, fd);
+
+      info("reexecuting self\n");
+      execv(argv[0], argv);
+
+      error("while trying to reexec self, trying to continue\n");
+      sigreexec_catched = false;
+    }
+
     /* Minimal error handling. */
     if (sd < 0)
     {
@@ -812,15 +962,15 @@ main(int argc, char *argv[])
       close(fd);
       free(prog.name);
 
-      release_childs(childs);
+      kindergarten_destroy(kg);
 
       /* Invoke it. */
       if (prog.filename)
       {
         info("invoking '%s'\n", prog.filename);
-        set_progname(prog.argv[0], argc, argv);
+        set_progname(prog.argv[0], argc, argv, -1);
 
-	booster.api->booster_init(prog.argv[0], booster.state);
+	boosters_init(boosters, prog.argv[0]);
 
 	launch_process(&prog);
       }
@@ -839,7 +989,11 @@ main(int argc, char *argv[])
 	child.sock = sd;
 	child.name = prog.name;
 
-	assign_child_slot(childs, &child);
+	if (!kindergarten_insert_child(kg, &child))
+	{
+	  invoked_send_fake_exit(child.sock);
+	  close(child.sock);
+	}
       }
       else
 	close(sd);
