@@ -1,7 +1,7 @@
 /*
- * $Id: prog.c 12989 2007-08-01 22:17:57Z guillem $
+ * $Id$
  *
- * Copyright (C) 2005, 2006, 2007 Nokia Corporation
+ * Copyright (C) 2005, 2006, 2007, 2008 Nokia Corporation
  *
  * Authors: Guillem Jover <guillem.jover@nokia.com>
  *
@@ -85,7 +85,7 @@ set_process_name(const char *progname)
 #endif
 
 void
-set_progname(char *progname, int argc, char **argv, int copy_index)
+set_progname(const char *progname, int argc, char **argv, int copy_index)
 {
   int argvlen = 0;
   int proglen = strlen(progname) + 1;
