@@ -1,7 +1,7 @@
 /*
- * $Id: prog.h 6793 2006-09-28 00:35:39Z guillem $
+ * $Id$
  *
- * Copyright (C) 2005, 2006 Nokia Corporation
+ * Copyright (C) 2005, 2006, 2007, 2008 Nokia Corporation
  *
  * Authors: Guillem Jover <guillem.jover@nokia.com>
  *
@@ -30,7 +30,7 @@ typedef struct
   int options;
   int argc;
   char **argv;
-  char *filename;
+  const char *filename;
   char *name;
   int prio;
   entry_t entry;
@@ -39,7 +39,7 @@ typedef struct
 void load_main(prog_t *prog);
 void print_prog_env_argv(prog_t *prog);
 
-void set_progname(char *progname, int argc, char **argv);
+void set_progname(const char *progname, int argc, char **argv, int copy_index);
 
 #endif
 
